#!/bin/sh
#
# AUTHOR:
#   Glenn O Larsen - Modified by Torsten (ivc)
###############################################################################
# http://www.linuxforums.org/forum/linux-programming-scripting/38474-bash-arrays.html
# http://articles.techrepublic.com.com/5100-1035_11-5820685.html?part=rss&tag=feed&subj=tr
# http://www.tech-recipes.com/bourne_shell_scripting_tips635.html
# http://steve-parker.org/sh/functions.shtml
# http://tldp.org/LDP/Bash-Beginners-Guide/html/sect_07_01.html
#
# Start program by reboot, or: "launchctl load /System/Library/LaunchDaemons/com.ivancover.lookup.plist"

# Global variables
LOCAL_PATH="/Applications/LookUp.app"
LAST_CALL_ID=0

# Set bin files
BIN_SQLITE="$LOCAL_PATH/bin/sqlite3"
BIN_ALERT="$LOCAL_PATH/LookDisplay.sh"
BIN_WC="$LOCAL_PATH/bin/wc"

# Sqlite settings
DATABASE_CALL_HISTORY="/var/mobile/Library/CallHistory/call_history.db"
DATABASE_ADDRESS_BOOK="/var/mobile/Library/AddressBook/AddressBook.sqlitedb"
DATABASE_LOOKUPS="$LOCAL_PATH/lookups.db"

# Queries
SQL_GET_LATEST_ROWID='SELECT ROWID from call WHERE flags = 4 AND duration = 0 AND id = -1 ORDER BY ROWID DESC LIMIT 1'
SQL_GET_LATEST_NUMBER='SELECT address from call WHERE flags = 4 AND duration = 0 AND id = -1 ORDER BY ROWID DESC LIMIT 1'

# Debug options
DEBUG=0

# Local testing
#DATABASE_CALL_HISTORY="/media/iphone/var/root/Library/CallHistory/call_history.db"
#BIN_SQLITE="/usr/bin/sqlite3"

### Main
main()
{
        # Get last call ID when starting the application
        LAST_CALL_ID=`sqliteQuery "$DATABASE_CALL_HISTORY" "$SQL_GET_LATEST_ROWID"`

        # Debug
        if [ "$DEBUG" == "1" ]; then echo "Launching background job..."; fi

        # Start daemon loop
        while true; do

                # Get last ID from call log
                CALL_ID=`sqliteQuery "$DATABASE_CALL_HISTORY" "$SQL_GET_LATEST_ROWID"`

                if [ "$DEBUG" == "1" ]; then  CALL_ID=`date`; fi

                # Check information - NB: Some times the query returns blank, not sure why...
                if [ "$CALL_ID" != "" ]; then

                        # If the ID we got from query is something else than the LAST_CALL_ID, we want to display the name
                        if [ "$LAST_CALL_ID" != "$CALL_ID" ]; then

                                # Debug
                                if [ "$DEBUG" == "1" ]; then echo "`date +'%F %H:%M'` 1: Setting LAST_CALL_ID ($LAST_CALL_ID) to CALL_ID ($CALL_ID)"; fi

                                # Set new call ID to the last call id
                                LAST_CALL_ID=$CALL_ID

                                # New unknown call, get the number from database
                                NUMBER=`sqliteQuery "$DATABASE_CALL_HISTORY" "$SQL_GET_LATEST_NUMBER"`

                                # Debug
                                if [ "$DEBUG" == "1" ]; then NUMBER="370`date +%S`000"; fi
                                if [ "$DEBUG" == "1" ]; then echo "`date +'%F %H:%M'` 2: Found number: $NUMBER"; fi

                                # Look it up
                                if [ -n "$NUMBER" ]; then

                                        # Remove all blanks in number
                                        NUMBER=`echo $NUMBER | sed "s/ *//g"`

                                        # Check if the number exists in the address book
                                        NUMBER_TO_SEARCH=`echo "$NUMBER" | sed -E "s/(.)/%\1/g"`
                                        ADDRESS_BOOK_UID=`sqliteQuery "$DATABASE_ADDRESS_BOOK" "SELECT UID FROM ABMultiValue WHERE value LIKE '$NUMBER_TO_SEARCH'"`

                                        # Debug
                                        if [ "$DEBUG" == "1" ]; then echo "`date +'%F %H:%M'` 3: Address book search string: $SQL_ADDRESS_BOOK_SEARCH, result: $ADDRESS_BOOK_UID"; fi

                                        # If it doesn't exists in the address book, we can continue
                                        if [ "$ADDRESS_BOOK_UID" == "" ]; then

                                                # Search in 1881
                                                NAME=`lookupnumber $NUMBER`

                                                # Debug
                                                if [ "$DEBUG" == "1" ]; then echo "`date +'%F %H:%M'` 4: Name from search: $NAME"; fi

                                                # Check if we got the name...
                                                if [ "$NAME" != "" ]; then
                                                        # ... we got it, display information to the user
                                                        DATE=`date +"%e %b %H:%M"`
                                                        displaymsg "Opplysningen 1881
Tapte anrop:" "$DATE:
$NAME - $NUMBER
"
                                                        INSERT=`sqliteQuery "$DATABASE_LOOKUPS" "INSERT INTO looklog VALUES(NULL,'$NAME', '$NUMBER', '$DATE')"`
                                                else
                                                        # ... no name, display information
                                                        DATE=`date +"%e %b %H:%M"`
                                                        displaymsg "Opplysningen 1881
Tapte anrop:" "$DATE:
$NUMBER - Ingen treff
"
                                                        INSERT=`sqliteQuery "$DATABASE_LOOKUPS" "INSERT INTO looklog VALUES(NULL,'Ingen treff', '$NUMBER', '$DATE')"`
                                                fi

                                        fi

                                fi

                        fi

                fi

                # Only check every 5 seconds, so we don't use to much CPU/battery
                sleep 8
        done
}

### Execute an SQLite command, returns results
sqliteQuery()
{
        echo `$BIN_SQLITE "$1" "$2"`;
}

### Search for name in 1881
lookupnumber()
{
	# Remove leading +47 and/or 0047 at the beginning (if its in the string)
	PHONENUMBER=`echo $1 | sed "s/\+47//g"`
	PHONENUMBER=`echo $PHONENUMBER | sed "s/^0047//g"`

	# Check length of string
	PHONENUMBER_LENGTH=`echo $PHONENUMBER | $BIN_WC -c | sed "s/ *//g"`

	# If phone number is not 8 + 1 (newline of echo) chars long, we skip checking it...
	if [ "$PHONENUMBER_LENGTH" != "9" ]; then
		echo ""
		return
	fi

	# Check if lookup before
	LOOKUP_EXIST=`sqliteQuery "$DATABASE_LOOKUPS" "SELECT name FROM lookups WHERE number LIKE '$PHONENUMBER'"`

	if [ "$LOOKUP_EXIST" != "" ]; then
		echo $LOOKUP_EXIST
		return
        fi

	# Look up the number in 1881.no - We use the ip, since DNS is not always stable
	DATA=`curl -s -m 10 "http://139.112.16.75/?Query=$PHONENUMBER"|grep -A 2 '<h3 class'|cut -d'"' -f4|head -3|tail -1`

	# Do we have text string left?
	if [ "$DATA" != "" ]; then

		# Replace some characters
		DATA=`replaceHTMLEntities "$DATA"`

		# Insert into db
		INSERT=`sqliteQuery "$DATABASE_LOOKUPS" "INSERT INTO lookups VALUES('$PHONENUMBER','$DATA')"`

		# Return result
		echo $DATA
		return
	fi

	# Return blank
	echo ""
}

### Replace HTML entities
replaceHTMLEntities()
{
	# Replace HTML entities
	STR=`echo "$1" | sed "s/&amp;/&/g"`
	STR=`echo "$STR" | sed "s/&#192;//g"`
	STR=`echo "$STR" | sed "s/&#193;//g"`
	STR=`echo "$STR" | sed "s/&#194;//g"`
	STR=`echo "$STR" | sed "s/&#195;//g"`
	STR=`echo "$STR" | sed "s/&#196;//g"`
	STR=`echo "$STR" | sed "s/&#197;//g"`
	STR=`echo "$STR" | sed "s/&#198;//g"`
	STR=`echo "$STR" | sed "s/&#199;//g"`
	STR=`echo "$STR" | sed "s/&#200;//g"`
	STR=`echo "$STR" | sed "s/&#201;//g"`
	STR=`echo "$STR" | sed "s/&#202;//g"`
	STR=`echo "$STR" | sed "s/&#203;//g"`
	STR=`echo "$STR" | sed "s/&#204;//g"`
	STR=`echo "$STR" | sed "s/&#205;//g"`
	STR=`echo "$STR" | sed "s/&#206;//g"`
	STR=`echo "$STR" | sed "s/&#207;//g"`
	STR=`echo "$STR" | sed "s/&#208;//g"`
	STR=`echo "$STR" | sed "s/&#209;//g"`
	STR=`echo "$STR" | sed "s/&#210;//g"`
	STR=`echo "$STR" | sed "s/&#211;//g"`
 	STR=`echo "$STR" | sed "s/&#212;//g"`
	STR=`echo "$STR" | sed "s/&#213;//g"`
	STR=`echo "$STR" | sed "s/&#214;//g"`
	STR=`echo "$STR" | sed "s/&#216;//g"`
	STR=`echo "$STR" | sed "s/&#217;//g"`
	STR=`echo "$STR" | sed "s/&#218;//g"`
	STR=`echo "$STR" | sed "s/&#219;//g"`
	STR=`echo "$STR" | sed "s/&#220;//g"`
	STR=`echo "$STR" | sed "s/&#221;//g"`
	STR=`echo "$STR" | sed "s/&#222;//g"`
	STR=`echo "$STR" | sed "s/&#223;//g"`
	STR=`echo "$STR" | sed "s/&#224;//g"`
	STR=`echo "$STR" | sed "s/&#225;//g"`
	STR=`echo "$STR" | sed "s/&#226;//g"`
	STR=`echo "$STR" | sed "s/&#227;//g"`
	STR=`echo "$STR" | sed "s/&#228;//g"`
	STR=`echo "$STR" | sed "s/&#229;//g"`
	STR=`echo "$STR" | sed "s/&#230;//g"`
	STR=`echo "$STR" | sed "s/&#231;//g"`
	STR=`echo "$STR" | sed "s/&#232;//g"`
	STR=`echo "$STR" | sed "s/&#233;//g"`
	STR=`echo "$STR" | sed "s/&#234;//g"`
	STR=`echo "$STR" | sed "s/&#235;//g"`
	STR=`echo "$STR" | sed "s/&#236;//g"`
	STR=`echo "$STR" | sed "s/&#237;//g"`
	STR=`echo "$STR" | sed "s/&#238;//g"`
	STR=`echo "$STR" | sed "s/&#239;//g"`
	STR=`echo "$STR" | sed "s/&#240;//g"`
	STR=`echo "$STR" | sed "s/&#241;//g"`
	STR=`echo "$STR" | sed "s/&#242;//g"`
	STR=`echo "$STR" | sed "s/&#243;//g"`
	STR=`echo "$STR" | sed "s/&#244;//g"`
	STR=`echo "$STR" | sed "s/&#245;//g"`
	STR=`echo "$STR" | sed "s/&#246;//g"`
	STR=`echo "$STR" | sed "s/&#248;//g"`
	STR=`echo "$STR" | sed "s/&#249;//g"`
	STR=`echo "$STR" | sed "s/&#250;//g"`
	STR=`echo "$STR" | sed "s/&#251;//g"`
	STR=`echo "$STR" | sed "s/&#252;//g"`
	STR=`echo "$STR" | sed "s/&#253;//g"`
	STR=`echo "$STR" | sed "s/&#254;//g"`
	STR=`echo "$STR" | sed "s/&#255;//g"`

	echo "$STR"
}


### Display result until tap
displaymsg()
{
	echo "$2" >> /Applications/LookUp.app/msg.txt
	msg=`cat /Applications/LookUp.app/msg.txt`
	if [ "$VENDOR" = "apple" ]; then
		display=`ps xa|grep -e alert|grep Opplysningen|grep -v grep|sed 's/^ *//g'|cut -d" " -f1`
		if [ -n "$display" ]; then
			kill $display
		fi
		$BIN_ALERT "$1" "$msg" &
	else
		echo "$1 - $2"
	fi
}

# Start program
main
